package BCD::Media;

use File::Copy::Recursive qw(dircopy pathrm);
use strict;
use File::Copy;
use File::stat;
use Data::Dumper;
use MDK::Common;

use BCD::Common qw(:DEFAULT $isoconf $wd $name $arch $version $repo $builddir);

our @ISA = qw(Exporter);
our @EXPORT = qw(main_media);

my $LOG="MEDIA -";
my $color = "magenta";

my $mediainfo_sourcedir;
if (! $isoconf->{mediainfo}{fullpath}) {
	$mediainfo_sourcedir = $isoconf->{mediainfo}{fullpath};
} else {
	$mediainfo_sourcedir = "$repo/$version/$arch/$isoconf->{mediainfo}{mediainfo_dir}";
}

sub create_media {
    print_color("$LOG Create the media dir in $builddir", $color);
    foreach my $media (@{$isoconf->{media}{list}}) {
	create_needed_dir("$builddir/media/$media->{destmedia}");
    }
}

sub show_size {
	print_color("$LOG size of the builddir", $color);
	system("du -sh $builddir");
}

sub list_media {
    print_color("$LOG all media are listed there:", $color);
    foreach my $media (@{$isoconf->{media}{list}}) {
    $media->{name} or next;
    print "
name: $media->{name}
    desc: $media->{desc}
    mediadir: $media->{mediadir}
    destmedia: $media->{destmedia}
    pubkey: $media->{pubkey}\n";
	foreach my $files (@{$media->{input}}) {
		$files->{file} or next;
		print "list of wanted: $files->{file}\n";
 	}
	foreach my $files (@{$media->{exclude}}) {
		$files->{file} or next;
		print "list of exclusion: $files->{file}\n";
 	}
   }
}

sub copy_pubkey {
    my ($source_key, $destmedia) = @_;
    print_color("$LOG copy $source_key to the $builddir/media/$destmedia/media_info directory", $color);
    create_needed_dir("$builddir/media/$destmedia/media_info/");
    if (! -f $source_key) {
	print_color("$LOG can't find the $source_key", $color);
	exit;
    }
    system("cp -vf $source_key $builddir/media/$destmedia/media_info/");
}

sub use_gendistrib {
	print_color("$LOG gendistrib --mediacfg $builddir/$isoconf->{iso}{mediacfg} $builddir", $color);
	system("gendistrib --mediacfg $builddir/$isoconf->{iso}{mediacfg} $builddir");
}

sub copy_rpm {
	my ($source_dir, $dest_dir) = @_;
	system ("cp -a $source_dir/*.rpm $dest_dir"); 
}

sub create_mediacfg {
    print_color("$LOG $builddir/$isoconf->{iso}{mediacfg}", $color);
    create_needed_dir("$builddir/$isoconf->{mediainfo}{mediainfo_dir}");
    my $media_cfg = "$builddir/$isoconf->{iso}{mediacfg}";
    open my $MEDIACFG, ">$media_cfg";
    print $MEDIACFG "[media_info]\n";
    print $MEDIACFG "version=$version\n";
    print $MEDIACFG "mediacfg_version=2\n";
    print $MEDIACFG "branch=$isoconf->{iso}{branch}\n";
    print $MEDIACFG "arch=$arch\n";
    print $MEDIACFG "minor=$isoconf->{iso}{minor}\n";
    print $MEDIACFG "subversion=$isoconf->{iso}{subversion}\n";
    if ($isoconf->{mediainfo}{suppl}{todo} eq "yes") {
        print $MEDIACFG "suppl=1\n";
    }
    if ($isoconf->{mediainfo}{askmedia}{todo} eq "yes") {
	print $MEDIACFG "askmedia=1\n";
    }
    if ($isoconf->{mediainfo}{xmlinfo}{todo} eq "yes") {
	print $MEDIACFG "xml-info=1\n";
    }
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	print $MEDIACFG "
[$media->{destmedia}]
synthesis=$media->{destmedia}/media_info/synthesis.hdlist.cz
pubkey=$media->{destmedia}/media_info/pubkey
desc='$media->{desc}'
\n";
##hdlists=../$media->{destmedia}/media_info/hdlist.cz
        }
    close $MEDIACFG;

}

# import from MDK::common
#sub cat_ { my @l = map { my $F; open($F, '<', $_) ? <$F> : () } @_; wantarray() ? @l : join '', @l }

# based on a similar functions in pkgs.pm
sub get_list_from_CAT {
    my ($file, $cat, $poids) = @_;
    my (%rates, @need_to_copy);
    my (@l);
    local $_;
    foreach (cat_($file)) {
        s/#.*//; # comments
        my ($indent, $data) = /(\s*)(.*)/;
        next if !$data; # skip empty lines
	
        @l = grep { $_->[0] < length $indent } @l;

        my @m = @l ? @{$l[-1][1]} : ();
        my ($t, $flag, @l2);
        while ($data =~
               /^((
                   [1-5]
                   |
                   (?:            (?: !\s*)? [0-9A-Z_]+(?:".*?")?)
                   (?: \s*\|\|\s* (?: !\s*)? [0-9A-Z_]+(?:".*?")?)*
                  )
                  (?:\s+|$)
                 )(.*)/x) { #@")) {
            ($t, $flag, $data) = ($1,$2,$3);
            while ($flag =~ s,^\s*(("[^"]*"|[^"\s]*)*)\s+,$1,) {}
	    push @m, $flag;
	    push @l2, [ length $indent, [ @m ] ];
	    $indent .= $t;
        }
        if ($data) {
            # has packages on same line
            my ($rates, $flags) = partition { /^\d$/ } @m;
            foreach my $name (split ' ', $data) {
                if (member($cat, @$flags)) {
		    if ("@$rates" ge "$poids") {
			push @need_to_copy, $name;
		    } else {
			print "exclude $name of $cat (@$rates < $poids)\n";
		    }
                    next;
                }
            }
            push @l, @l2;
        } else {
            push @l, [ $l2[0][0], $l2[-1][1] ];
        }
    }
    \%rates, \@need_to_copy;
}


my $listcopy = "/tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}/$isoconf->{tocopy_file}";
sub generate_list_of_rpm_2_copy {
    create_needed_dir("/tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}");
    my @toget;my @toexclude;my $extra;
    
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	foreach my $input (@{$media->{input}}) {
	    $input->{file} or next;
	    -f $input->{file} or print_color("$LOG i can not find the $input->{file}, remove this list or fix...", $color);
	    foreach my $tohave (cat_($input->{file})) {
		if ($tohave =~ /^\s/ || $tohave =~ /^\#/) {
		    next;
		} elsif ($tohave =~ /^\S*\s\d$/) {
		    my ($section, $poids) = $tohave =~ /^(\S*)\s(\d)/;
		    print "$LOG get packages from $section using weight of $poids\n";
	            my ($rates, $need_to_copy) = get_list_from_CAT("$builddir/$isoconf->{mediainfo}{mediainfo_dir}/rpmsrate", $section, $poids);
		    foreach my $data (@{$need_to_copy}) {
			push @toget, $data;
		    }
		    next;
		} else {
		    chomp($tohave);
		    push @toget, $tohave;
		    next;
		}
	    }
	}
	foreach my $exclude (@{$media->{exclude}}) {
	    $exclude->{file} or next;
	    -f $exclude->{file} or print_color("$LOG i can not find the $exclude->{file}, remove this list or fix...", $color);
	    foreach my $totest (cat_($exclude->{file})) {
		if ($totest =~ /^\s/ || $totest =~ /^\#/) {
		    next;
		} else {
		    chomp($totest);
		    push @toexclude, $totest;
		    next;
		}
	    }
	}
#	print "$LOG $media, $toget, $toexclude";
    	create_list($media, \@toget, \@toexclude, $extra);
	@toexclude = "";
	@toget = "";
    }
}

sub create_list {
    my ($media, $wanted, $exclude, $extra) = @_;
    my $extra_media;
    # Main media is mandatory for all other media !
    # we will use only this media to calculte the main list
	if ($media->{name} eq "Main") {
	$extra_media = "--media \"Main\"" ;
	} else { 
	$extra_media = "--media \"Main,$media->{name}\"";
	}
    my $opt_sug;
    # check the suggests option
    if ($media->{nosuggests} eq "yes") {
	$opt_sug = "--no-suggests";
    }
    # list of wanted RPM per media name
    my $list = "/tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}/$isoconf->{tocopy_file}_todo_$media->{name}";
    # urpmi error file
    my $urpmi_error = "/tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}/$isoconf->{tocopy_file}_urpmi_error";
    # remove main32 and main32 updates for x86_64 iso
    my $exclude_media;
    if ($arch eq "x86_64") {
    # $media->{name} eq "restricted") {
	   $exclude_media = "--excludemedia 'Main32,Main32 Updates'";
   }
    my $tested_wanted;
    my $tested_exclude;
    foreach (@{$wanted}) { $tested_wanted = $tested_wanted . " $_" }
    foreach (@{$exclude}) { $tested_exclude = $tested_exclude . " --skip $_"  };
    # urpmi optipns
    my $urpmi_options = "--no-verify-rpm -v --keep --auto -a --ignoresize --test --nolock $exclude_media";
    my $urpmi_extra_opt = "$opt_sug $extra_media";
    my $cmd = "sudo urpmi $urpmi_options $urpmi_extra_opt --use-distrib $repo/$version/$arch/ --urpmi-root /tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}/ $tested_wanted $tested_exclude > $list 2> $urpmi_error";
    system($cmd);
    # urpmi error found, so we have to remove some entry in the list of wanted packages
    my @toremove;
    if (-f $urpmi_error) {
	foreach my $entry (cat_($urpmi_error)) {
	    my ($tor) = $entry =~ /\w+\s\w+\s\w+\s(.*)\n/;
	    $tor or next;
	    push @toremove, $tor;
	}
	my %already_seen; my @final_list;
	@already_seen{@toremove} = ();
	foreach my $object (@{$wanted}) {
	    if (exists $already_seen{$object}) { print_color("$LOG i cant find this package: $object, remove it from list", "red"); }
	    push(@final_list, $object) unless exists $already_seen{$object};
	}
	my $final_list;
	foreach (@final_list) {
		chomp($_);
		$final_list = $final_list . " $_";
	}
        my $cmd2 = "sudo urpmi $urpmi_options $urpmi_extra_opt --use-distrib $repo/$version/$arch/ --urpmi-root /tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}/ $final_list $tested_exclude > $list";
        print_color("$LOG $cmd2", $color);
	system($cmd2);
    }
#    if ($media->{name} eq "Main") { print "No extra from Media $media->{name}\n" } else { remove_extra_from_main($list, $wanted, $media); }
}

# remove unwanted RPM taken from main, while wanted from other media
sub remove_extra_from_main {
	my ($list, $wanted, $media) = @_;
	# remove package in main from $wanted in $list (wrongly added)
#for i in `cat ~/bcd/lists/input_contrib` ; do grep $i tocopy_plop_todo_Contrib | grep "\/main/.*/$i" ; done
	
	my $list_to_remove = "";
	# remove deps in main for those RPM
	my $urpmi_options = "--no-verify-rpm --auto -a --ignoresize --test --nolock";
#	my $cmd = "sudo urpmi $opt_sug $urpmi_options $extra_media --use-distrib $repo/$version/$arch/ --urpmi-root /tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}/ $list_to_remove > $list.deps_to_remove";
#	print "\n $LOG $cmd \n";
	# remove all those RPM from the $list
	# $list.deps_to_remove and list_to_remove
}

sub concatenate_files {
    print_color("$LOG create the final list: $listcopy", $color);
    system("cat /tmp/$isoconf->{nameid}-$isoconf->{version}-$isoconf->{arch}/$isoconf->{tocopy_file}_todo_* | uniq | sort > $listcopy");
}

sub target_size {
	print_color("$LOG calculate the size of all packages", $color);
	my $total;my $pkg_info;
	foreach my $pkg (cat_($listcopy)) {
		chomp($pkg);
		if (-f "$pkg") {
		$pkg_info = stat($pkg);
		$total = ($total+$pkg_info->size);
		next;
		} else { next; }
	}
	my $human_total = int(($total/1024)/1024);
	print_color("$LOG Size of all RPMS: $human_total Mo", $color);
	my $left = ($isoconf->{iso}{size} - $human_total);
	# i need to add the install/ directory size
	if ("$isoconf->{iso}{size}" ge "$human_total") {
		print_color("$LOG all you data will fit on the CD/DVD, extra size: $left", $color);
	} else {
		print_color("$LOG all packages will not fit on $isoconf->{iso}{size}, i need $left Mo", 'red');
	}
}

sub count_nb_to_copy {
    my ($file) = @_;
    my $lines = 0;
    open(FILE, $file) or die "Can't open `$file': $!";
    while (sysread FILE, my $buffer, 4096) {
	if ($buffer =~ /^install/ || $buffer =~ /^\-/ || $buffer =~ /^More/ || $buffer =~ /^Install/ ) {
	    next;
	} else {
	    $lines += ($buffer =~ tr/\n//);
	}
    }
    close FILE;
    return $lines;
}

sub parse_copy_link {
    my ($wichaction) = @_;
    my $count;my $action;
    if ($wichaction =~ /link/) {
	    $action = "ln -sf";
    } else {
	$action = "cp -a";
    }
    my $total = count_nb_to_copy($listcopy);
    print_color("$LOG copy all wanted packages to $builddir/XXXX", $color);
    foreach my $tocopy (cat_($listcopy)) {
	# use only needed info from urpmi Output (maybe there is some issue with other languages than EN)
	if ($tocopy =~ /^install/ || $tocopy =~ /^\-/ || $tocopy =~ /^More/ || $tocopy =~ /^Install/ ) {
            next;
        } else {
	    # create a basic progress report
	    $count++;
	    my $p = int(($count*100)/$total);
	    if ($p == 20 || $p == 40 || $p == 60 || $p == 80) { print "$p% -"; }
	    chomp($tocopy);
	    foreach my $media (@{$isoconf->{media}{list}}) {
		$media->{name} or next;
		if ($tocopy =~ /media\/$media->{mediadir}/) {
		    # print "$LOG $action $tocopy TO $builddir/media/$media->{destmedia}\n";
		    -f $tocopy or next;
		    system("$action $tocopy $builddir/media/$media->{destmedia}");
		    last;
		}
	    }
	}
	next;
    }
}

sub copy_filedeps {
	print_color("$LOG copy $mediainfo_sourcedir/$isoconf->{mediainfo}{filedeps} in $builddir/$isoconf->{mediainfo}{mediainfo_dir}", $color);
	-f "$mediainfo_sourcedir/$isoconf->{mediainfo}{filedeps}" or print "$LOG missing filedeps\n";
	system("cp -vf $mediainfo_sourcedir/$isoconf->{mediainfo}{filedeps} $builddir/$isoconf->{mediainfo}{mediainfo_dir}");
}

sub copy_rpmsrate {
	print_color("$LOG copy $mediainfo_sourcedir/$isoconf->{mediainfo}{rpmsrate} in $builddir/$isoconf->{mediainfo}{mediainfo_dir}", $color);
	-f "$mediainfo_sourcedir/$isoconf->{mediainfo}{rpmsrate}" or print "$LOG missing rpmsrate\n";
	system("cp -vf $mediainfo_sourcedir/$isoconf->{mediainfo}{rpmsrate} $builddir/$isoconf->{mediainfo}{mediainfo_dir}");
}

sub copy_compssusers {
	print_color("$LOG copy $mediainfo_sourcedir/$isoconf->{mediainfo}{compssusers} in $builddir/$isoconf->{mediainfo}{mediainfo_dir}", $color);
	-f "$mediainfo_sourcedir/$isoconf->{mediainfo}{compssusers}" or print "$LOG missing compssusers\n";
	system("cp -vf $mediainfo_sourcedir/$isoconf->{mediainfo}{compssusers} $builddir/$isoconf->{mediainfo}{mediainfo_dir}");
}

sub use_genhdlist2 {
	foreach my $media (@{$isoconf->{media}{list}}) {
		    $media->{name} or next;
		print_color("$LOG build synthesys in $builddir/media/$media->{destmedia}", $color);
		system("genhdlist2 --quiet --clean --no-hdlist --file-deps $builddir/media/media_info/$isoconf->{medianfo}{filedeps} $builddir/media/$media->{destmedia}");
	}
}

# to FIX
sub copy_all_pubkey {
    foreach my $media (@{$isoconf->{media}{list}}) {
	$media->{name} or next;
	if ($media->{pubkey} eq "") {
	    print_color("$LOG no pubkey to copy (usefull when mixing main/contrib packages)", $color);
	} else {
	# media/ dir will be added in the copy_pubkey functions
	    print_color("$LOG $repo/$version/$arch/media/$media->{mediadir}/$media->{pubkey}, $media->{destmedia}", $color);
	    copy_pubkey("$repo/$version/$arch/media/$media->{mediadir}/$media->{pubkey}", "$media->{destmedia}");
	}
	next;
    }
}

sub main_media {
    list_media;
    create_media;
    copy_all_pubkey;

    create_mediacfg;

    copy_filedeps;
    copy_rpmsrate;
    copy_compssusers;

    generate_list_of_rpm_2_copy;
    concatenate_files;
    target_size;
    parse_copy_link("link");
#    use_genhdlist2;
    use_gendistrib;
    show_size;
}

1;

